class ProposalAction < ActiveRecord::Base
  has_many :reasons, class_name: ProposalReason, inverse_of: :action
  belongs_to :zone
  # přidá score jako sloupec
  scope :with_score, -> { joins(:reasons).
    select('proposal_actions.*, SUM(proposal_reasons.score) as score').
    group(:proposal_action_id).order('score DESC, priority, catid, updated_at') 
  }
  include OsmHelper
  
  def self.update_or_create_by(options)
    action = self.find_or_create_by(options)
    action.inicializuj_dle_catid
    action.save
    action
  end

  def inicializuj_dle_catid
    parametry = self.catid.split('|')
    case parametry.first
      # JOIN GUIDEPOST + OSM GUIDEPOST ############
    when 'jgog-ng' # new guidepost
      raise "Predelano na ActionJoinOsmGuidepostByKctref"
    when 'jgog-eg'
      raise "Predelano na ActionJoinOsmGuidepostByKctref"
    when 'jsog-ng'
      raise "Predelano na ActionJoinSeznamAndOsmGuidepost"
    when 'jsog-eg'
      raise "Predelano na ActionJoinSeznamAndOsmGuidepost"
    when 'jkot-nt','jtot-nt' # new trail
      raise "Predelano na ActionJoinKctOsmTrail"
    when 'jkot-et','jtot-et' # existing trail
      raise "Predelano na ActionJoinKctOsmTrail"
      ### NAME GUIDEPOST ############################
      ### name guideposts
    when 'ng'
      raise 'Predelano pomoci ActionSetGuidepostName'
    when 'lgfog' # locate guidepost from osm_guidepost
      raise 'Predelano pomoci ActionSetGuidepostLocation'
    when 'chfgtog' # changes from guidepost to osm_guidepost
      raise 'Predelano pomoci ActionSetOsmGuidepostTags'
    when 'iogtot' # Insert Osm_Guidepost To Osm_Trail
      raise 'Predelano pomoci ActionInsertOsmGuidepostToOsmTrail'
    when 'colt' # Colorize trail
      raise "Predelano na ActionJoinSeznamAndOsmGuidepost"
    when 'vsg', 'vsd'
      _,osmid,gap_str = parametry
      osm_trail = OsmTrail.find_by(osmid: osmid)
      raise "OSM trail s osmnid #{osmid} neexistuje" if osm_trail.nil?
      raise "Gap string are not recognized" unless /\[\[([0-9\.]*)\s*,\s*([0-9\.]*)\s*\],\s*\[([0-9\.]*)\s*,\s*([\d\.]*)\]\]/.match(gap_str)
      gap = [[$1.to_f,$2.to_f],[$3.to_f,$4.to_f]]
      guessed_zone = osm_trail.guess_zone
      
      self.assign_attributes( # inicializuj všechny atributy
        action: linkify("Ručně uzavřít na ot:#{osmid} mezeru délky #{Map.distance_string(*gap)}"),
        short_action: {'vsg' => 'mezeru ručně zavřít', 'vsd' => 'odbočku ručně zkontrolovat'}[parametry.first],
        category: 'close gap',
        url: ApplicationController.helpers.josm_zoom_to_points_with_zoom(2,*gap),
        priority: priority_from_kctref(guessed_zone),
        zone_id: guessed_zone
      )
    when 'dogfot' # Delete Osm_Guidepost From Osm_Trail
      _,osm_trail_osmid,osm_guidepost_osmid = parametry # načti parametry
      ot = OsmTrail.find_by(osmid: osm_trail_osmid)
      og = OsmGuidepost.find_by(osmid: osm_guidepost_osmid); 
      raise "OSM trasa ot:#{osm_trail_osmid} nebyla nalezena!" if ot.nil?
      raise "OSM rozcestník og:#{osm_guidepost_osmid} nebyl nalezen!" if og.nil?
      url=josm_delete_osm_guidepost_from_osm_trail(ot, og)
      raise "URL je nil" if url.nil?
      
      self.assign_attributes( # inicializuj všechny atributy
        action: linkify("Vyjmi rozcestník og:#{og.osmid} z ot:#{ot.osmid} (<a href=\"#{josm_load_objects(ot,og)}\">viz v JOSM</a>)"),
        short_action: 'vyjmout OGs z OT',
        category: 'delete membership',
        url: url,
        priority: priority_from_kctref(og.kctref),
        zone_id: guessed_guidepost_zone_by_kctref(og.kctref)
      )
    when 'igtot' # Insert guidepost to trail
      raise 'Predelano pomoci ActionInsertGuidepostToTrail'
    when 'cgj' # Create guidepost on junction
      _, kctref,lat,lon = parametry
      kg = KctGuidepost.find_by_kctref(kctref)
      raise "Rozcestník kg:#{kctref} nebyl nalezen!" if kg.nil?
      
      self.assign_attributes( 
        action: linkify("Přidat chybějící rozcestník k kg:#{kctref} přímo na křižovatku"), 
        short_action: "přidat G na křižovatku",
        category: 'create guidepost',
        url: "/guideposts/new?guidepost[kctref]=#{kctref}&guidepost[lat]=#{lat}&guidepost[lon]=#{lon}&guidepost[nazev]=#{kg.nazev_recased}"
      )  
      # url: ApplicationController.helpers.josm_add_node([lat,lon],{
      #   information: 'guidepost',
      #   tourism: 'information',
      #   name: kct_guidepost.nazev_recased,
      #   ref: kct_guidepost.kctref,
      #   fixme: 'position somewhere near this crossing, its existence guessed from info on other guideposts',
      #   hiking: 'yes',
      #   operator: 'cz:kčt'
      # }),
    else 
      raise "Akce nerozpoznána na základě catid: #{catid}"
    end
  end
    
  # normálně se nepoužívá, ale je možno tímto akci přímo manuálně prověst, vrací boolean podle toho jestli se akce povedla
  def doit
    raise 'Predelano na Action subclasses'
  end
  
  # celkové sore akce - udává jak pravděpodobné je že akce je správná
  def sum_score
    reasons.sum(:score)
  end
  
  # textová interpretace číselného skóre
  def text_score
    case sum_score
    when -999..-1 then 'chybná'
    when 0..49 then 'nejistá'
    when 50..99 then 'ke zvážení'
    when 100..149 then 'celkem jistě správná'
    when 150..249 then 'jistě správná'
    when 250..999 then 'naprosto jistě správná'
    else '????'
    end
  end
      
  # akce ke kterým není žádný důvod (např. proto že už byly provedeny)
  scope :unused, -> { includes(:reasons).where(proposal_reasons: {id: nil}) }
  scope :zavrhnut, -> { where(zavrhnut: true) }
  scope :nezavrhnut, -> { where(zavrhnut: false) }
  scope :catid_begins, ->(string) { where("catid like ?", string+'%') }
      
  # smaž akce ke kterým není žádný důvod (např. proto že už byly provedeny)
  def self.destroy_unused
    ProposalAction.unused.nezavrhnut.destroy_all  # mozna je mozno mazat i nepouzivane zavrzene ale zatim takhle
  end
  
  # url které akci ihned vykoná
  def doit_url
    if self.instance_of? ProposalAction
      raise "ProposalAction #{id} nemá url" if url.nil?
      url+'&doit=true'
    else
      url
    end
  end
  
  # má v sobě url více kroků
  def more_steps_url?
    /\|/.match(url)
  end
  
  def urls
    url.split('|')
  end
  
  # doplní text o odkazy na objekty typu kct:PS001 nebo qa:14578 které jsou v něm zmíněny
  def self.linkify(text)
    text.gsub!(/((kg:)([[:upper:]]{2}[[:digit:]]{3}))/, '<a href="/kct_guideposts/kctref-\3">\1</a>')
    text.gsub!(/((qa:|rg:)([[:upper:]]{2}[[:digit:]]{3}))/, '<a href="/guideposts/kctref-\3">\1</a>')
    text.gsub!(/((rg:)([[:digit:]]+))/, '<a href="/guideposts/\3">\1</a>')
    text.gsub!(/((og:)([[:digit:]]+))/, '<a href="/osm_guideposts/osmid-\3">\1</a>')
    text.gsub!(/((sg:)([[:digit:]]+))/, '<a href="/sez_guideposts/\3">\1</a>')
    text.gsub!(/((kt:)([[:digit:]]+))/, '<a href="/kct_trails/kctref-\3">\1</a>')
    text.gsub!(/((rt:)([[:digit:]]+))/, '<a href="/trails/kctref-\3">\1</a>')
    text.gsub!(/((ot:)([[:digit:]]+))/, '<a href="/osm_trails/osmid-\3">\1</a>')
    text.html_safe
  end
  def linkify(text); ProposalAction.linkify(text) end  # prasarnicka
  
  def doit; raise 'Nutno implementovat v podtřídě' end
  
  def short_action2; nil end  # ProposalAction v základu musí mít short_action přímo v databázi
  
  private
  def priority_from_kctref(kctref)
    case kctref
    when /^(PS|PJ)/ then 10
    else 999
    end
  end
  def guessed_trail_zone_by_kctref(trail_kctref)
    kct_trail_or_trail = KctTrail.find_by(kctref: trail_kctref) || Trail.find_by(kctref: trail_kctref)
    return 'XX' if kct_trail_or_trail.nil?
    return kct_trail_or_trail.guess_zone
  end
  def guessed_guidepost_zone_by_kctref(guidepost_kctref)
    return 'XX' if guidepost_kctref.nil?
    return guidepost_kctref[0..1]
  end
end
    
